import logging
from typing import Any

import sqlparse

logger = logging.getLogger(__name__)


def format_sql_query(query: str, **options: Any) -> str:
    try:
        return sqlparse.format(query, **options)
    except Exception as e:
        logger.debug(f"Exception:{e} while formatting query '{query}'.")
        return query



import re

import pydantic

from datahub.configuration.common import ConfigModel, ConfigurationError


class BigQueryBaseConfig(ConfigModel):
    rate_limit: bool = pydantic.Field(
        default=False, description="Should we rate limit requests made to API."
    )
    requests_per_min: int = pydantic.Field(
        default=60,
        description="Used to control number of API calls made per min. Only used when `rate_limit` is set to `True`.",
    )

    temp_table_dataset_prefix: str = pydantic.Field(
        default="_",
        description="If you are creating temp tables in a dataset with a particular prefix you can use this config to set the prefix for the dataset. This is to support workflows from before bigquery's introduction of temp tables. By default we use `_` because of datasets that begin with an underscore are hidden by default https://cloud.google.com/bigquery/docs/datasets#dataset-naming.",
    )

    sharded_table_pattern: str = pydantic.Field(
        default="((.+)[_$])?(\\d{4,10})$",
        description="The regex pattern to match sharded tables and group as one table. This is a very low level config parameter, only change if you know what you are doing, ",
    )

    @pydantic.validator("sharded_table_pattern")
    def sharded_table_pattern_is_a_valid_regexp(cls, v):
        try:
            re.compile(v)
        except Exception as e:
            raise ConfigurationError(
                f"sharded_table_pattern configuration pattern is invalid. The exception was: {e}"
            )
        return



# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# fmt: off
from .....schema_classes import CorpGroupEditableInfoClass
from .....schema_classes import CorpGroupInfoClass
from .....schema_classes import CorpUserCredentialsClass
from .....schema_classes import CorpUserEditableInfoClass
from .....schema_classes import CorpUserInfoClass
from .....schema_classes import CorpUserStatusClass
from .....schema_classes import GroupMembershipClass
from .....schema_classes import InviteTokenClass


CorpGroupEditableInfo = CorpGroupEditableInfoClass
CorpGroupInfo = CorpGroupInfoClass
CorpUserCredentials = CorpUserCredentialsClass
CorpUserEditableInfo = CorpUserEditableInfoClass
CorpUserInfo = CorpUserInfoClass
CorpUserStatus = CorpUserStatusClass
GroupMembership = GroupMembershipClass
InviteToken = InviteTokenClass

# fmt: on



import itertools
import shlex
from typing import List

import requests


def _make_curl_command(
    session: requests.Session, method: str, url: str, payload: str
) -> str:
    fragments: List[str] = [
        "curl",
        *itertools.chain(
            *[
                ("-X", method),
                *[("-H", f"{k!s}: {v!s}") for (k, v) in session.headers.items()],
                ("--data", payload),
            ]
        ),
        url,
    ]
    return " ".join(shlex.quote(fragment) for fragment in fragments)



import enum
from datetime import datetime, timedelta, timezone
from typing import Any, Dict

import pydantic
from pydantic.fields import Field

from datahub.configuration.common import ConfigModel
from datahub.metadata.schema_classes import CalendarIntervalClass


@enum.unique
class BucketDuration(str, enum.Enum):
    DAY = CalendarIntervalClass.DAY
    HOUR = CalendarIntervalClass.HOUR


def get_time_bucket(original: datetime, bucketing: BucketDuration) -> datetime:
    """Floors the timestamp to the closest day or hour."""

    if bucketing == BucketDuration.HOUR:
        return original.replace(minute=0, second=0, microsecond=0)
    else:  # day
        return original.replace(hour=0, minute=0, second=0, microsecond=0)


def get_bucket_duration_delta(bucketing: BucketDuration) -> timedelta:
    if bucketing == BucketDuration.HOUR:
        return timedelta(hours=1)
    else:  # day
        return timedelta(days=1)


class BaseTimeWindowConfig(ConfigModel):
    bucket_duration: BucketDuration = Field(
        default=BucketDuration.DAY,
        description="Size of the time window to aggregate usage stats.",
    )

    # `start_time` and `end_time` will be populated by the pre-validators.
    # However, we must specify a "default" value here or pydantic will complain
    # if those fields are not set by the user.
    end_time: datetime = Field(default=None, description="Latest date of usage to consider. Default: Last full day in UTC (or hour, depending on `bucket_duration`)")  # type: ignore
    start_time: datetime = Field(default=None, description="Earliest date of usage to consider. Default: Last full day in UTC (or hour, depending on `bucket_duration`)")  # type: ignore

    @pydantic.validator("end_time", pre=True, always=True)
    def default_end_time(
        cls, v: Any, *, values: Dict[str, Any], **kwargs: Any
    ) -> datetime:
        return v or get_time_bucket(
            datetime.now(tz=timezone.utc)
            + get_bucket_duration_delta(values["bucket_duration"]),
            values["bucket_duration"],
        )

    @pydantic.validator("start_time", pre=True, always=True)
    def default_start_time(
        cls, v: Any, *, values: Dict[str, Any], **kwargs: Any
    ) -> datetime:
        return v or (
            values["end_time"]
            - get_bucket_duration_delta(values["bucket_duration"]) * 2
        )

    @pydantic.validator("start_time", "end_time")
    def ensure_timestamps_in_utc(cls, v: datetime) -> datetime:
        if v.tzinfo != timezone.utc:
            raise ValueError(
                'timezone is not UTC; try adding a "Z" to the value e.g. "2021-07-20T00:00:00Z"'
            )
        return v



from dataclasses import dataclass
from typing import Any, Dict, List, Optional

from datahub.ingestion.api.committable import CommitPolicy
from datahub.ingestion.api.common import PipelineContext
from datahub.ingestion.api.ingestion_job_state_provider import (
    IngestionJobStateProvider,
    IngestionJobStateProviderConfig,
    JobId,
    JobStateFilterType,
    JobStateKey,
    JobStatesMap,
)
from datahub.metadata.schema_classes import DatahubIngestionRunSummaryClass

#
# Common type exports
#
JobId = JobId
JobStateKey = JobStateKey
JobStateFilterType = JobStateFilterType

#
# Reporting state specific types
#
ReportingJobStateType = DatahubIngestionRunSummaryClass
ReportingJobStatesMap = JobStatesMap[ReportingJobStateType]


class IngestionReportingProviderConfig(IngestionJobStateProviderConfig):
    pass


@dataclass()
class IngestionReportingProviderBase(IngestionJobStateProvider[ReportingJobStateType]):
    """
    The base class(non-abstract) for all reporting state provider implementations.
    This class is implemented this way as a concrete class is needed to work with the registry,
    but we don't want to implement any of the functionality yet.
    """

    def __init__(self, name: str, commit_policy: CommitPolicy = CommitPolicy.ALWAYS):
        super(IngestionReportingProviderBase, self).__init__(name, commit_policy)

    @classmethod
    def create(
        cls, config_dict: Dict[str, Any], ctx: PipelineContext, name: str
    ) -> "IngestionJobStateProvider":
        raise NotImplementedError("Sub-classes must override this method.")

    def get_previous_states(
        self,
        state_key: JobStateKey,
        last_only: bool = True,
        filter_opt: Optional[JobStateFilterType] = None,
    ) -> List[ReportingJobStatesMap]:
        raise NotImplementedError("Sub-classes must override this method.")

    def commit(self) -> None:
        raise NotImplementedError("Sub-classes must override this method.")



from typing import List, Optional

import datahub.emitter.mce_builder as builder
from datahub.configuration.common import ConfigModel
from datahub.ingestion.api.common import PipelineContext
from datahub.ingestion.transformer.base_transformer import (
    BaseTransformer,
    SingleAspectTransformer,
)
from datahub.metadata.schema_classes import StatusClass


class MarkDatasetStatusConfig(ConfigModel):
    removed: bool


class MarkDatasetStatus(BaseTransformer, SingleAspectTransformer):
    """Transformer that marks status of each dataset."""

    ctx: PipelineContext
    config: MarkDatasetStatusConfig

    def __init__(self, config: MarkDatasetStatusConfig, ctx: PipelineContext):
        super().__init__()
        self.ctx = ctx
        self.config = config

    def aspect_name(self) -> str:
        return "status"

    def entity_types(self) -> List[str]:
        return ["dataset"]

    @classmethod
    def create(cls, config_dict: dict, ctx: PipelineContext) -> "MarkDatasetStatus":
        config = MarkDatasetStatusConfig.parse_obj(config_dict)
        return cls(config, ctx)

    def transform_aspect(
        self, entity_urn: str, aspect_name: str, aspect: Optional[builder.Aspect]
    ) -> Optional[builder.Aspect]:
        assert aspect is None or isinstance(aspect, StatusClass)
        status_aspect: StatusClass = aspect if aspect else StatusClass(removed=None)
        status_aspect.removed = self.config.removed
        return status_aspect  # type: ignore
